/*--------------------------------------------------------------------------*/
/* Copyright 2022-2023 NXP                                                  */
/*                                                                          */
/* NXP Confidential. This software is owned or controlled by NXP and may    */
/* only be used strictly in accordance with the applicable license terms.   */
/* By expressly accepting such terms or by downloading, installing,         */
/* activating and/or otherwise using the software, you are agreeing that    */
/* you have read, and that you agree to comply with and are bound by, such  */
/* license terms. If you do not agree to be bound by the applicable license */
/* terms, then you may not retain, install, activate or otherwise use the   */
/* software.                                                                */
/*--------------------------------------------------------------------------*/

#include "common.h"

#include <mcuxClToolchain.h>
#include <mcuxClSession.h> // Interface to the entire mcuxClSession component
#include <mcuxCsslFlowProtection.h> // Code flow protection
#include <mcuxClPsaDriver.h>
#include <mcuxClCore_Examples.h>
#include <mcuxClExample_ELS_Helper.h>

/**
 * @brief Example value for RSA key pairs (non-encrypted DER encoding format).
 */
static const ALIGNED uint8_t keyBuffer[] = {
  //SEQUENCE{
  0x30,
  0x82, 0x04, 0xA4,
  //version           Version,  -- 0
  0x02,
  0x01,
  0x00,
  //modulus           INTEGER,  -- 256-byte
  0x02,
  0x82, 0x01, 0x01,
  0x00, 0xBE, 0xD8, 0xFF, 0x2D, 0xBC, 0xE9, 0x6E, 0xCB, 0x7C, 0xB6, 0x86, 0x86, 0x6D, 0x01, 0x98,
  0x41, 0x49, 0x38, 0x06, 0xCA, 0x50, 0x8F, 0x5C, 0xF0, 0x3A, 0x02, 0x90, 0x90, 0x5B, 0xC5, 0x1A,
  0xCC, 0xE6, 0x69, 0x17, 0xF2, 0x53, 0x58, 0xC0, 0x94, 0x93, 0xEA, 0x57, 0x2B, 0xC1, 0x09, 0x69,
  0x46, 0x81, 0xD3, 0x15, 0x4C, 0xD5, 0x23, 0xBE, 0x32, 0x06, 0xB6, 0xD0, 0xEA, 0x30, 0xD3, 0xDD,
  0x65, 0x9B, 0xE8, 0xAC, 0xC7, 0x0B, 0x4C, 0xA5, 0x14, 0xE9, 0x01, 0x9E, 0x4E, 0xEE, 0x2F, 0x57,
  0x8A, 0x64, 0x71, 0x59, 0xC9, 0x4C, 0x11, 0xE2, 0xE0, 0xEC, 0xC9, 0x96, 0x75, 0xF4, 0x92, 0xDF,
  0x1E, 0x84, 0x78, 0xBD, 0xC4, 0x3C, 0xC1, 0x03, 0x8D, 0x3C, 0x4E, 0x70, 0x25, 0x22, 0x0A, 0x15,
  0x0A, 0xFF, 0x9E, 0x2B, 0x45, 0x0C, 0x72, 0x11, 0x0A, 0xE5, 0x4B, 0x3C, 0xCB, 0x8A, 0x80, 0x3C,
  0x41, 0x42, 0xFE, 0x78, 0x34, 0xF0, 0x1A, 0x55, 0x37, 0x1B, 0x7D, 0x3A, 0xEE, 0x38, 0x25, 0x58,
  0x52, 0x27, 0x75, 0x9E, 0x59, 0x41, 0xFA, 0x43, 0x11, 0x92, 0xB9, 0x70, 0x17, 0x1D, 0x4B, 0x11,
  0xDA, 0xE0, 0xF5, 0xB7, 0x77, 0x48, 0x93, 0x4E, 0x3B, 0x68, 0x60, 0x08, 0x86, 0x57, 0xD6, 0x61,
  0xBF, 0x4A, 0x31, 0x41, 0xFA, 0x11, 0xFB, 0x3A, 0x90, 0x3A, 0x22, 0xB8, 0xE0, 0x38, 0x27, 0xB9,
  0x25, 0x8D, 0x0E, 0xDE, 0x8A, 0xDC, 0x65, 0x04, 0x7B, 0xDF, 0x4A, 0xA0, 0x5F, 0x78, 0x8F, 0x7E,
  0xC5, 0x66, 0xFF, 0x85, 0x33, 0x73, 0x06, 0x23, 0x24, 0x39, 0x1F, 0x66, 0x26, 0x18, 0x16, 0x53,
  0x30, 0x2E, 0x24, 0xC4, 0x92, 0x39, 0x13, 0x14, 0x98, 0x53, 0x84, 0xEA, 0x99, 0xDC, 0x40, 0x57,
  0x30, 0xC4, 0x2F, 0xE7, 0x89, 0xB6, 0x69, 0x5D, 0x60, 0x0F, 0x4B, 0x1D, 0x66, 0x54, 0x22, 0x8D,
  0xB1,
  //publicExponent    INTEGER,  -- 3-byte
  0x02,
  0x03,
  0x01, 0x00, 0x01,
  //privateExponent   INTEGER,  -- 256-byte
  0x02,
  0x82, 0x01, 0x00,
  0x03, 0x31, 0x2D, 0xA2, 0x23, 0x60, 0xEE, 0x28, 0x50, 0xD2, 0x31, 0x66, 0xD4, 0x87, 0x97, 0x21,
  0xD9, 0xBF, 0xD4, 0xFB, 0xE8, 0xF9, 0x5F, 0x8F, 0x14, 0x66, 0xA5, 0x94, 0xB1, 0xE1, 0x96, 0x9A,
  0x00, 0x6D, 0x42, 0x92, 0xC6, 0xDF, 0xC5, 0xA7, 0x81, 0x35, 0x30, 0x0C, 0x67, 0x22, 0xA9, 0x54,
  0x48, 0xBD, 0xF1, 0xED, 0xC2, 0x8F, 0x8C, 0xA7, 0x59, 0x38, 0x66, 0x94, 0x7C, 0x33, 0x96, 0xFB,
  0x93, 0x98, 0xD7, 0xDC, 0x4E, 0x6D, 0x3E, 0x42, 0x49, 0x71, 0x6C, 0x51, 0xEF, 0xFC, 0x47, 0xBA,
  0x64, 0xB0, 0x06, 0xAB, 0x43, 0x6E, 0xA0, 0x7D, 0x02, 0xF3, 0x8E, 0x3A, 0x33, 0xD8, 0x89, 0xB5,
  0xD6, 0x21, 0x0F, 0x8A, 0x2A, 0xE5, 0x4E, 0xE7, 0x66, 0x5A, 0x53, 0x82, 0xDE, 0x27, 0xB8, 0x04,
  0x0D, 0x3E, 0xCF, 0xDA, 0x21, 0xA3, 0xFE, 0x1D, 0x50, 0xB4, 0xD4, 0xF3, 0xFC, 0x96, 0xE4, 0xD5,
  0xBF, 0xD0, 0x76, 0x79, 0xC6, 0xCC, 0x23, 0x44, 0xD5, 0xFA, 0x28, 0xCE, 0x5E, 0x21, 0xB0, 0x2C,
  0x4E, 0xE4, 0x8E, 0x01, 0x19, 0x4E, 0xAB, 0x60, 0x1B, 0x42, 0x3A, 0x7B, 0x90, 0x1C, 0x8B, 0x73,
  0x98, 0xAB, 0xFE, 0xF1, 0xA7, 0xC2, 0x1E, 0x97, 0x9F, 0x80, 0x4E, 0xCB, 0xAB, 0xBF, 0x01, 0x9E,
  0x6E, 0x24, 0x51, 0x3E, 0x01, 0x4E, 0xAA, 0xC0, 0x9B, 0xD5, 0xD9, 0x6F, 0x85, 0x9C, 0xAE, 0x75,
  0xCE, 0x7D, 0x05, 0xF9, 0x58, 0xFF, 0xB4, 0x87, 0xAA, 0xC4, 0xFA, 0xA9, 0xEB, 0xC2, 0x72, 0x09,
  0x77, 0x7F, 0x1D, 0x89, 0x4B, 0x23, 0x77, 0x90, 0x73, 0x4F, 0xB1, 0xBF, 0xB1, 0x5E, 0x26, 0xED,
  0x58, 0x32, 0x23, 0x91, 0x08, 0xBE, 0xC9, 0xD7, 0x6F, 0x9C, 0x32, 0xB9, 0x74, 0x29, 0xF8, 0x70,
  0x8A, 0x84, 0x18, 0xAF, 0xEC, 0x0A, 0xA6, 0xF0, 0x81, 0x98, 0x98, 0x6C, 0xE6, 0x4B, 0xF5, 0xDF,
  //prime1            INTEGER,  -- 128-byte
  0x02,
  0x81, 0x81,
  0x00, 0xC1, 0x26, 0x5C, 0x6B, 0xD6, 0x5C, 0x5D, 0x57, 0xBF, 0xA9, 0x60, 0x2E, 0xCA, 0x66, 0x30,
  0x46, 0xD8, 0x2A, 0x16, 0x1E, 0xEA, 0xB3, 0xD7, 0xF2, 0x15, 0xAB, 0x39, 0xD4, 0x9B, 0xFC, 0x4A,
  0xB3, 0x67, 0x8A, 0xC0, 0x17, 0xE7, 0x43, 0x6B, 0x3D, 0xF1, 0xB3, 0xA3, 0x31, 0x13, 0x21, 0x3B,
  0x98, 0x53, 0x14, 0x73, 0x7D, 0x10, 0xEB, 0x72, 0x3E, 0x2E, 0x08, 0xC8, 0xC9, 0x57, 0xC7, 0x45,
  0xDF, 0x5D, 0xD5, 0x6E, 0xF4, 0xAB, 0x99, 0x66, 0x8C, 0x5F, 0x48, 0xF0, 0xD4, 0x95, 0xF2, 0xEB,
  0xCB, 0x73, 0x7F, 0x70, 0x69, 0x6E, 0x81, 0x5D, 0x86, 0xAC, 0xFB, 0xBD, 0x02, 0x97, 0x5B, 0xD3,
  0xEB, 0x3A, 0x4D, 0xBC, 0x51, 0xF5, 0xA9, 0x9B, 0xC0, 0xB4, 0xFC, 0x6C, 0xF9, 0xE2, 0xC6, 0xCA,
  0x5A, 0x42, 0x6B, 0x82, 0x10, 0xD8, 0x47, 0x8C, 0xFC, 0x9E, 0x4B, 0x11, 0x8A, 0xF3, 0xE1, 0x4E,
  0x23,
  //prime2            INTEGER,  -- 128-byte
  0x02,
  0x81, 0x81,
  0x00, 0xFC, 0xF2, 0xDB, 0xEF, 0x1A, 0x9E, 0x4E, 0xD5, 0x74, 0x1D, 0xF0, 0x08, 0x58, 0xD4, 0xEB,
  0xDE, 0x88, 0x45, 0xAD, 0xC0, 0xD3, 0xA6, 0xA2, 0x36, 0x93, 0xE7, 0x3B, 0x68, 0x51, 0x18, 0x63,
  0x16, 0x79, 0x8D, 0x4F, 0x08, 0x2E, 0xE1, 0x7E, 0xDC, 0x6F, 0x41, 0x53, 0x64, 0xF1, 0xE0, 0x3A,
  0xDF, 0xD4, 0x7D, 0x98, 0xF8, 0x93, 0x23, 0xEE, 0x52, 0xC4, 0x2E, 0x31, 0x50, 0xFA, 0x68, 0x73,
  0xA0, 0x93, 0xAF, 0xCF, 0xA4, 0x21, 0xAE, 0x43, 0x0A, 0x3F, 0x97, 0xCA, 0x58, 0x61, 0x60, 0xB7,
  0xE5, 0x78, 0x35, 0xD8, 0xAC, 0x6F, 0x11, 0xBE, 0x96, 0xEB, 0xA9, 0xA9, 0x0C, 0x5A, 0xE4, 0x63,
  0x48, 0xBD, 0x00, 0x26, 0xEB, 0xD7, 0xDE, 0x6A, 0xBD, 0x0B, 0xB8, 0xA3, 0x8A, 0x34, 0x12, 0x88,
  0xC9, 0x84, 0x4D, 0xD3, 0xA9, 0x0A, 0x5E, 0xED, 0xA9, 0x2F, 0x1E, 0x2B, 0x09, 0x2D, 0x10, 0x70,
  0x1B,
  //exponent1         INTEGER,  -- 128-byte
  0x02,
  0x81, 0x81,
  0x00, 0x82, 0xAB, 0x62, 0x21, 0x2E, 0x5F, 0x44, 0x62, 0xE5, 0xEE, 0x3F, 0x7C, 0xC8, 0x3F, 0x03,
  0xF0, 0x19, 0xB3, 0xB7, 0x4D, 0x69, 0x39, 0x0C, 0x21, 0xE1, 0xD8, 0xFA, 0x01, 0xC5, 0x19, 0x94,
  0xAB, 0xF4, 0xA3, 0xA0, 0xBB, 0x4B, 0x20, 0x88, 0x3F, 0xDA, 0xF1, 0xCD, 0xB8, 0x98, 0x99, 0x86,
  0x08, 0xD2, 0x43, 0xE6, 0xB1, 0xB8, 0xAD, 0xA0, 0x97, 0x42, 0x6B, 0x7C, 0xF3, 0x01, 0xE8, 0x75,
  0x73, 0xDC, 0xB6, 0x55, 0x1F, 0x3F, 0xAC, 0x42, 0xFD, 0x3A, 0x45, 0x4D, 0x70, 0x74, 0x95, 0x68,
  0x42, 0x36, 0xBC, 0x03, 0x9F, 0xC0, 0x3B, 0xD2, 0xBB, 0x16, 0xF2, 0x23, 0xF7, 0xC9, 0xD0, 0x3C,
  0xF9, 0x49, 0x73, 0x67, 0xB1, 0x07, 0x02, 0x9C, 0xB5, 0x6D, 0x7B, 0xCC, 0x79, 0xED, 0x9A, 0xD1,
  0x30, 0xE8, 0xF8, 0x74, 0x80, 0xD2, 0xE0, 0xED, 0x17, 0xC6, 0x3B, 0x40, 0xFE, 0x01, 0x69, 0xEE,
  0x83,
  //exponent2         INTEGER,  -- 128-byte
  0x02,
  0x81, 0x81,
  0x00, 0xB0, 0xBE, 0x7D, 0xA1, 0x10, 0x07, 0x67, 0xEC, 0x4C, 0x6B, 0x92, 0xCA, 0x32, 0x4F, 0xEC,
  0xD4, 0x1C, 0x82, 0x1B, 0x8B, 0xAE, 0x18, 0x34, 0x26, 0x50, 0xA8, 0x74, 0xE1, 0x4A, 0x30, 0xF1,
  0x23, 0xC6, 0x21, 0x50, 0x04, 0xD6, 0xC5, 0x27, 0xA0, 0x9D, 0x78, 0x96, 0xED, 0xE4, 0xF8, 0x9A,
  0x0A, 0xC6, 0x6E, 0x50, 0x51, 0xF8, 0x76, 0x55, 0xD3, 0xAD, 0x52, 0xDD, 0x90, 0xC8, 0xB7, 0xED,
  0x7B, 0x59, 0x56, 0xB2, 0x8E, 0xEC, 0x1D, 0xD8, 0xA8, 0x33, 0x91, 0x3B, 0x89, 0x0F, 0xD9, 0xC6,
  0x05, 0x68, 0x3E, 0xAF, 0xBC, 0xA5, 0x0B, 0x50, 0x12, 0x22, 0x6E, 0xF5, 0x39, 0x35, 0xD5, 0x79,
  0xEE, 0x5C, 0x69, 0xDB, 0xC8, 0x55, 0x99, 0x0B, 0x1A, 0x37, 0x33, 0x77, 0xCA, 0x5C, 0xE2, 0x4A,
  0x84, 0x0C, 0x97, 0x58, 0xFB, 0x37, 0xCC, 0xE6, 0xE1, 0x9D, 0x93, 0xC5, 0xDC, 0x6E, 0x89, 0x9A,
  0xDB,
  //coefficient       INTEGER,  -- 128-byte
  0x02,
  0x81, 0x80,
  0x66, 0xB2, 0x11, 0x6F, 0x95, 0xF8, 0x21, 0x42, 0xC3, 0xAE, 0x71, 0xBD, 0x49, 0x1D, 0x2E, 0xF9,
  0x8D, 0xE8, 0xEF, 0xBE, 0x98, 0xB3, 0xD2, 0x36, 0xD5, 0x34, 0x48, 0x2B, 0xF8, 0x3E, 0xB1, 0x85,
  0xF4, 0x87, 0x3B, 0x16, 0xD3, 0xEE, 0x2C, 0xCE, 0xA9, 0x05, 0xDB, 0x59, 0x0F, 0x73, 0x5C, 0x33,
  0xEA, 0x70, 0xF7, 0xF3, 0xF6, 0x88, 0x7C, 0xC1, 0x1D, 0x87, 0xDD, 0xA0, 0x33, 0x1C, 0xAE, 0x6D,
  0x08, 0xA4, 0x5C, 0x3F, 0x41, 0x5C, 0x1C, 0x18, 0x7C, 0xB8, 0x45, 0x53, 0x57, 0x9A, 0x91, 0x1F,
  0x41, 0xF9, 0x1D, 0x9A, 0x9A, 0x1E, 0x1D, 0xFC, 0x75, 0x36, 0x42, 0xE5, 0x6B, 0x21, 0x9C, 0x67,
  0xF2, 0x66, 0xFB, 0x62, 0xC4, 0xE9, 0xF8, 0x51, 0x1D, 0xD9, 0xBD, 0xB8, 0x25, 0xD8, 0xE5, 0x60,
  0x9D, 0x3C, 0xA1, 0xDE, 0x05, 0xDC, 0x29, 0x2C, 0x4A, 0x55, 0xED, 0xF6, 0xAD, 0xF2, 0xC4, 0xDF
};

/**
 * @brief Input message to be signed
 */
static const ALIGNED uint8_t message[3] = {
  0x61, 0x62, 0x63
};

/**
 * @brief Signature
 */
MCUX_CSSL_ANALYSIS_START_PATTERN_EXTERNAL_MACRO()
static ALIGNED uint8_t signature[PSA_SIGN_OUTPUT_SIZE(PSA_KEY_TYPE_RSA_KEY_PAIR, 2048u, PSA_ALG_RSA_PKCS1V15_SIGN_BASE)];
MCUX_CSSL_ANALYSIS_STOP_PATTERN_EXTERNAL_MACRO()

/* Example of RSA PKCS1V15 signature generation and verification for 2048-bit key and SHA-256 */
MCUXCLEXAMPLE_FUNCTION(mcuxClPsaDriver_rsa_PKCS1V15_sign_verify_message_example)
{

  /** Initialize ELS, Enable the ELS **/
  if(!mcuxClExample_Els_Init(MCUXCLELS_RESET_DO_NOT_CANCEL))
  {
      return MCUXCLEXAMPLE_STATUS_ERROR;
  }

  /*
   * Sign message: PKCS1V15, SHA_256
   */

  /* Set up PSA key attributes. */
  const psa_key_attributes_t sign_attributes = {
    .core = {                                                                                                               // Core attributes
      .type = PSA_KEY_TYPE_RSA_KEY_PAIR,                                                                                    // RSA key pair
      .bits = 2048u,                                                                                                        // Key bits
      .lifetime = PSA_KEY_LIFETIME_FROM_PERSISTENCE_AND_LOCATION(PSA_KEY_LIFETIME_VOLATILE, PSA_KEY_LOCATION_LOCAL_STORAGE),// Volatile (RAM), Local Storage for key pair
      .id = 0U,                                                                                                             // ID zero
      .policy = {
        .usage = PSA_KEY_USAGE_SIGN_MESSAGE,                                                                                // Key may be used to signing a message
        .alg = PSA_ALG_RSA_PKCS1V15_SIGN(PSA_ALG_SHA_256),                                                                  // RSA PKCS#1 v1.5 signature with hashing
        .alg2 = PSA_ALG_NONE},
      .flags = 0U},                                                                                                         // No flags
    .domain_parameters = NULL,
    .domain_parameters_size = 0U};

  size_t signature_length;

  psa_status_t sign_status = psa_driver_wrapper_sign_message(
    &sign_attributes,                               //const psa_key_attributes_t *attributes,
    keyBuffer,                                      //const uint8_t *key_buffer,
    sizeof(keyBuffer),                              //size_t key_buffer_size,
    PSA_ALG_RSA_PKCS1V15_SIGN(PSA_ALG_SHA_256),     //psa_algorithm_t alg,
    message,                                        //const uint8_t *input,
    sizeof(message),                                //size_t input_length,
    signature,                                      //uint8_t *signature,
    sizeof(signature),                              //size_t signature_size,
    &signature_length                               //size_t *signature_length
    );

  /* Check the return value */
  if(sign_status != PSA_SUCCESS)
  {
    return MCUXCLEXAMPLE_STATUS_ERROR;
  }

  /* Check the signature length */
  MCUX_CSSL_ANALYSIS_START_PATTERN_EXTERNAL_MACRO()
  if(signature_length != PSA_SIGN_OUTPUT_SIZE(PSA_KEY_TYPE_RSA_KEY_PAIR, 2048u, PSA_ALG_RSA_PKCS1V15_SIGN_BASE))
  MCUX_CSSL_ANALYSIS_STOP_PATTERN_EXTERNAL_MACRO()
  {
    return MCUXCLEXAMPLE_STATUS_ERROR;
  }

  /*
   * Verify message: PKCS1V15, SHA_256
   */

  /* Set up PSA key attributes. */
  const psa_key_attributes_t verify_attributes = {
    .core = {                                                                                                               // Core attributes
      .type = PSA_KEY_TYPE_RSA_KEY_PAIR,                                                                                    // RSA key pair
      .bits = 2048u,                                                                                                        // Key bits
      .lifetime = PSA_KEY_LIFETIME_FROM_PERSISTENCE_AND_LOCATION(PSA_KEY_LIFETIME_VOLATILE, PSA_KEY_LOCATION_LOCAL_STORAGE),// Volatile (RAM), Local Storage for key pair
      .id = 0U,                                                                                                             // ID zero
      .policy = {
        .usage = PSA_KEY_USAGE_VERIFY_MESSAGE,                                                                              // Key may be used for verify a message
        .alg = PSA_ALG_RSA_PKCS1V15_SIGN(PSA_ALG_SHA_256),                                                                  // RSA PKCS#1 v1.5 signature with hashing
        .alg2 = PSA_ALG_NONE},
      .flags = 0U},                                                                                                         // No flags
    .domain_parameters = NULL,
    .domain_parameters_size = 0U};

  psa_status_t verify_status = psa_driver_wrapper_verify_message(
    &verify_attributes,                         //const psa_key_attributes_t *attributes,
    keyBuffer,                                  //const uint8_t *key_buffer,
    sizeof(keyBuffer),                          //size_t key_buffer_size,
    PSA_ALG_RSA_PKCS1V15_SIGN(PSA_ALG_SHA_256), //psa_algorithm_t alg,
    message,                                    //const uint8_t *input,
    sizeof(message),                            //size_t input_length,
    signature,                                  //const uint8_t *signature,
    sizeof(signature)                           //size_t signature_length
    );

  /* Check the return value */
  if(verify_status != PSA_SUCCESS)
  {
    return MCUXCLEXAMPLE_STATUS_ERROR;
  }

  /* Return */
  return MCUXCLEXAMPLE_STATUS_OK;
}

